/*
 * bit_set.c
 *
 *  Created on: Oct 13, 2011
 *      Author: ibondarenko
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "e_malloc.h"
#include "bit_set.h"


/* ---------------- bit_set_getsize  ---------------------------------
 * returns the number of bytes + 1 for SIZE bits  
 */
size_t
bit_set_getsize(size_t size){

    return ( ((size % CHAR_BIT) != 0) ? size / CHAR_BIT + 2 : size / CHAR_BIT + 1 );
  
}


/* ---------------- bit_set_calloc  ---------------------------------
 * allocates SIZE bits memory  
 */
char*
bit_set_calloc(size_t size){

    return ( (char*) E_MALLOC (bit_set_getsize(size) ) );

}


/* ---------------- bit_set_free  ---------------------------------
 *  
 */
void
bit_set_free(char* bs) {
    free_if_not_null(bs);
}


/* ---------------- bit_set_bitset ---------------------------------
 *  sets POS bit to 1
 */
void
bit_set_bitset(char* bs, size_t pos) {

    char XXXmask = 1;
    XXXmask <<= ((pos) % CHAR_BIT);
    bs[(pos) / CHAR_BIT] |= XXXmask;

}


/* ---------------- bit_set_ifbitset  ---------------------------------
 * retruns true if POS bit is 1 
 */
unsigned char
bit_set_ifbitset(char* bs, size_t pos) {

    char XXXmask = 1;
    XXXmask <<= ((pos) % CHAR_BIT);
    return ((bs[(pos) / CHAR_BIT] & XXXmask) == XXXmask);

}


/* ---------------- bit_set_clean ---------------------------------
 *  sets POS bit to 0
 */
void
bit_set_clean(char* bs, size_t pos) {
   
    char XXXmask = 1;
    XXXmask <<= ((pos) % CHAR_BIT);
    bs[(pos) / CHAR_BIT] ^= XXXmask;
    
}




